/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include "driverlib.h"
#include "tmp116.h"
//*****************************************************************************
//! This example shows how to configure the I2C module as a master for
//! multi-byte transmission in interrupt driven mode. The address of the slave
//! module is set in this example.
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - I2C peripheral
//! - GPIO Port peripheral (for I2C pins)
//! - SCL2
//! - SDA
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - USCI_B0_VECTOR.
//
//*****************************************************************************
//*****************************************************************************
//
//Set the address for slave module. This is a 7-bit address sent in the
//following format:
//[A6:A5:A4:A3:A2:A1:A0:RS]
//
//A zero in the "RS" position of the first byte means that the master
//transmits (sends) data to the selected slave, and a one in this position
//means that the master receives data from the slave.
//
//*****************************************************************************
#define SLAVE_ADDRESS 0x48
//*****************************************************************************
//
//Specify number of bytes to be transmitted
//
//*****************************************************************************
unsigned char TXLENGTH = 1;

//Set Pointer to TMP116 Temperature Register
uint8_t transmitData[40] = {TMP116TEMPERATUREREADING};

uint8_t transmitCounter = 0;

unsigned char RXCOUNT = 2;

unsigned char receiveBuffer[10] = { 0x01, 0x01, 0x01, 0x01, 0x01,
                                    0x01, 0x01, 0x01, 0x01, 0x01};
unsigned char *receiveBufferPointer;
unsigned char receiveCount = 0;

int16_t temp;
float tempC;

//Define the data transmission and receive functions
void DataTransmit(unsigned char x);
void DataReceive(unsigned char y);

//Define the temperature float conversion function
void testfloatfunction();

//Main
void main()
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);
    //Set the internal pull-up resistors
    //Ports 4.2 and 4.1 are SCL/SDA, respectively
    GPIO_setAsInputPinWithPullUpResistor(
            GPIO_PORT_P4,
            GPIO_PIN1 + GPIO_PIN2
    );

    //Assign I2C pins to USCI_B1
    GPIO_setAsPeripheralModuleFunctionOutputPin(
            GPIO_PORT_P4,
            GPIO_PIN1 + GPIO_PIN2
    );

    //Initialize Master
    USCI_B_I2C_initMasterParam param = {0};
    param.selectClockSource = USCI_B_I2C_CLOCKSOURCE_SMCLK;
    param.i2cClk = UCS_getSMCLK();
    param.dataRate = USCI_B_I2C_SET_DATA_RATE_100KBPS;
    USCI_B_I2C_initMaster(USCI_B1_BASE, &param);

    //Specify slave address
    USCI_B_I2C_setSlaveAddress(USCI_B1_BASE,
                               SLAVE_ADDRESS
    );

    //Set Transmit mode
    USCI_B_I2C_setMode(USCI_B1_BASE,
                       USCI_B_I2C_TRANSMIT_MODE
    );

    //Enable I2C Module to start operations
    USCI_B_I2C_enable(USCI_B1_BASE);

    //Start Operations
    while(1)
    {
        //Load default configuration settings
        transmitData[0] = TMP116CONFIGURATION;
        transmitData[1] = TMP116CONFIGREGISTERCONV2;
        transmitData[2] = TMP116CONFIGREGISTERAVG0;
        //Transmit the Data
        DataTransmit(3);
        //Verify data written
        DataReceive(2);

        //Set Register Pointer for next read
        transmitData[0] = TMP116TEMPERATUREREADING;
        //Transmit Data
        DataTransmit(1);
        //Receive Data
        DataReceive(2);

        //Convert the temperature reading into a floating point value
        tempC = uint2floatconvertdata(receiveBuffer[0], receiveBuffer[1]);

        //Set Pointer to TMP116 Alert High Pointer and set to 30C
        transmitData[0] = TMP116ALERTHIGH;
        transmitData[1] = 0x1E;
        transmitData[2] = 0x00;
        //Transmit the Data
        DataTransmit(3);
        //Verify data written
        DataReceive(2);

        //Set Pointer to TMP116 Alert Low Pointer and set to 10C
        transmitData[0] = TMP116ALERTLOW;
        transmitData[1] = 0x0A;
        transmitData[2] = 0x00;
        //Transmit the Data
        DataTransmit(3);
        //Verify data written
        DataReceive(2);

        //Enable low power idle mode
        transmitData[0] = TMP116CONFIGURATION;
        transmitData[1] = TMP116SHUTDOWN;
        transmitData[2] = 0x00;
        //Transmit the Data
        DataTransmit(3);
        //Verify data written
        DataReceive(2);

        //Initiate temperature measurement &
        //Return to Shutdown
        transmitData[0] = TMP116CONFIGURATION;
        transmitData[1] = TMP116ONESHOT;
        transmitData[2] = 0x00;

        //Transmit the Data
        DataTransmit(3);
        //Verify data written
        DataReceive(2);

        //Wait for measurement to complete
        //refer to Table 7 TMP116 Datasheet
        __delay_cycles(5000);

        //Set Register Pointer for next read
        transmitData[0] = TMP116TEMPERATUREREADING;
        //Transmit Data
        DataTransmit(1);
        //Receive Data
        DataReceive(2);
        //Convert the temperature reading into a floating point value
        tempC = uint2floatconvertdata(receiveBuffer[0], receiveBuffer[1]);

        //verify temperature conversion function using examples from
        //datasheet Table 1
        testfloatfunction();
    }
    //******************************************************************************
    //
    //This is the USCI_B1 interrupt vector service routine.
    //
    //******************************************************************************
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
}
#pragma vector = USCI_B1_VECTOR;
__interrupt
#elif defined(__GNUC__)
__attribute__((interrupt(USCI_B1_VECTOR)))
#endif
void USCI_B1_ISR(void)
{
    switch(__even_in_range(UCB1IV,12))
    {
    case USCI_I2C_UCTXIFG:
    {
        //Check TX byte counter
        if(transmitCounter < TXLENGTH)
        {
            //Initiate send of character from Master to Slave
            USCI_B_I2C_masterSendMultiByteNext(USCI_B1_BASE,
                                               transmitData[transmitCounter]
            );

            //Increment TX byte counter
            transmitCounter++;
        }
        else
        {
            //Initiate stop only
            USCI_B_I2C_masterSendMultiByteStop(USCI_B1_BASE);

            //Clear master interrupt status
            USCI_B_I2C_clearInterrupt(USCI_B1_BASE,
                                      USCI_B_I2C_TRANSMIT_INTERRUPT);

            //Exit LPM0 on interrupt return
            __bic_SR_register_on_exit(LPM0_bits);
        }
        break;
    }
    case USCI_I2C_UCRXIFG:
    {
        //Decrement RX byte counter
        receiveCount--;
        if(receiveCount)
        {
            if(receiveCount == 1)
            {
                //Initiate end of reception -> Receive byte with NAK
                *receiveBufferPointer++ =
                        USCI_B_I2C_masterReceiveMultiByteFinish(
                                USCI_B1_BASE
                        );
            }
            else
            {
                //Keep receiving one byte at a time
                *receiveBufferPointer++ = USCI_B_I2C_masterReceiveMultiByteNext(
                        USCI_B1_BASE
                );
            }
        }
        else
        {
            //Receive last byte
            *receiveBufferPointer = USCI_B_I2C_masterReceiveMultiByteNext(
                    USCI_B1_BASE
            );
            USCI_B_I2C_disableInterrupt(USCI_B1_BASE,
                                        USCI_B_I2C_RECEIVE_INTERRUPT);
            USCI_B_I2C_clearInterrupt(USCI_B1_BASE,
                                      USCI_B_I2C_RECEIVE_INTERRUPT);
            __bic_SR_register_on_exit(LPM0_bits);
        }
        break;
    }
    }
}

//This is the data transmission function
void DataTransmit(unsigned char x) {
    //Set the byte length of data to be transmitted
    TXLENGTH = x;
    //Set Transmit mode
    USCI_B_I2C_setMode(USCI_B1_BASE,
                       USCI_B_I2C_TRANSMIT_MODE
    );
    //Enable transmit Interrupt
    USCI_B_I2C_clearInterrupt(USCI_B1_BASE,
                              USCI_B_I2C_TRANSMIT_INTERRUPT
    );
    USCI_B_I2C_enableInterrupt(USCI_B1_BASE,
                               USCI_B_I2C_TRANSMIT_INTERRUPT
    );

    //Delay between each transaction
    __delay_cycles(50);

    //Load TX byte counter
    transmitCounter = 1;

    //Initiate start and send first character
    USCI_B_I2C_masterSendMultiByteStart(USCI_B1_BASE,
                                        transmitData[0]
    );
    //Enter low power mode 0 with interrupts enabled.
    __bis_SR_register(LPM0_bits + GIE);
    __no_operation();
}

//This is the data receive function
void DataReceive(unsigned char y) {
    //Delay until transmission completes
//    while(USCI_B_I2C_isBusBusy(USCI_B1_BASE))
//    {
//        ;
//    }
    //Set the byte length of the data to be received
    RXCOUNT = y;
    //Set receive mode
    USCI_B_I2C_setMode(USCI_B1_BASE,
                       USCI_B_I2C_RECEIVE_MODE
    );

    //Enable master Receive interrupt
    USCI_B_I2C_enableInterrupt(USCI_B1_BASE,
                               USCI_B_I2C_RECEIVE_INTERRUPT
    );
    USCI_B_I2C_clearInterrupt(USCI_B1_BASE,
                              USCI_B_I2C_RECEIVE_INTERRUPT
    );

    receiveBufferPointer = (unsigned char *)receiveBuffer;
    receiveCount = RXCOUNT;

    //Initialize multi reception
    USCI_B_I2C_masterReceiveMultiByteStart(USCI_B1_BASE);

    //Enter low power mode 0 with interrupts enabled.
    __bis_SR_register(LPM0_bits + GIE);
    __no_operation();
}

//Test Temperature Float Function
void testfloatfunction() {
    const uint8_t HiByte[] = {0x80, 0xF3, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x32, 0x7F};
    const uint8_t LoByte[] = {0x00, 0x80, 0xF0, 0xFF, 0x00, 0x01, 0x10, 0x80, 0x80, 0x00, 0xFF};
    uint8_t i;
    float testC[12];
    for (i = 0; i<11; i++) {
        testC[i] = uint2floatconvertdata(HiByte[i], LoByte[i]);
        //avoid compiler warning for testC variable
        if (testC[i] < 30)
            __no_operation();
    }
}
